<?php
namespace Controller;

use Service\Traits\TResponse;
use Util\OpensslCrypt;
use Util\Utilities;
use Zeedhi\Framework\DTO;
use Service\Manager\Mongo\Manager;
use Service\DeleteCascadeService;
use Service\ConnectionService;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Zmart\DataSource\ZmartFilterCriteria;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Util\Environment;

class Connection extends Crud {

	use TResponse;

	protected $dataSourceName = 'connection';
	/** @var \Service\ConnectionService $connectionService */
	protected $connectionService;
	protected $environment;
	protected $mongoManager;

	public function __construct(Manager $organizationMongoManager, DeleteCascadeService $deleteCascadeService, ConnectionService $connectionService, Environment $environment) {
		parent::__construct($organizationMongoManager, $deleteCascadeService);
		$this->mongoManager = $organizationMongoManager;
        $this->mongoManager->createDatabase(\Util\Utilities::getOrganizationIdForMongo());
		$this->connectionService = $connectionService;
		$this->environment = $environment;
		$organizationMongoManager->createDatabase(\Util\Utilities::getOrganizationIdForMongo());
	}

    public function save(DTO\Request $request, DTO\Response $response)
    {
        $request = $this->buildDataSetIfRow($request);
        foreach ($request->getDataSet()->getRows() as &$row) {
            if(isset($row["oracleWalletUse"]) && $row["driverClass"] == "oracle.jdbc.driver.OracleDriver" && $row["oracleWalletUse"] == 'Y'){
                $validate = Utilities::validateWallet($row["dbUrl"]);
                if(!$validate){
                    throw new \Exception('ERROR_WALLET_SECURITY',1);
                }
            }
            $row["dbUrlSsl"] = OpensslCrypt::encrypt($row["dbUrl"]);
            $row["usernameSsl"] = OpensslCrypt::encrypt($row["username"]);
            $row["dbUrl"] = null;
            $row["username"] = null;
            if (isset($row["passwordNeedEncrypt"])) {
                $row["passwordSsl"] = $row["passwordNeedEncrypt"] ? OpensslCrypt::encrypt($row["password"]) : null;
                unset($row["passwordNeedEncrypt"]);
            }
        }
        $row["password"] = null;
        if (isset($row["passwordSsl"])) {
            $response->addDataSet(new DataSet('encryptedPassword', array($row["passwordSsl"], $row["passwordSsl"])));
        }else{
            $response->addDataSet(new DataSet('encryptedPassword', array("", "")));
        }
        $dataBefore = null;
        if(!empty($row['_id'])){
            $dataBefore = $this->findById($row['_id']);
        }
        parent::save($request, $response);
        $id = $response->getDataSets()[1]->getRows()[0]['_id'];
        $this->setLog($dataBefore,$id);
    }

    public function setLog($dataBefore,  $id)
    {
        $dataAfter = $this->findById($id);
        Utilities::setLog($dataBefore, $dataAfter, $this->dataSourceName, $id);
    }

    public function findById($id) {
        $filterCriteria = new FilterCriteria($this->dataSourceName, array(array(
            "columnName"=>"_id",
            "value" => $id,
            "operator" => "="
        )));

        return $this->mongoManager->findBy($filterCriteria)->getRows();
    }

	private function filter($params) {
        $filterCriteria = new FilterCriteria($this->dataSourceName, array());
        return $filterCriteria;
    }

    public function findData(DTO\Request $request, DTO\Response $response) {
    	$filterData = $this->filter($request->getParameters());
		$filterData->setDataSourceName($this->dataSourceName);
		$dataSet = $this->mongoManager->findBy($filterData);
		$response->addDataSet($dataSet);
		return $response;
    }

    public function findNext(DTO\Request $request, DTO\Response $response)
    {
    	$params = $request->getParameters();
        Utilities::convertDynamicFilterToArray($params);
        if(!Utilities::userCurrentIsTeknisa()){
            $params["dynamic_filter"]["IS_TEKNISA"] = [["relation"=>  "AND", "operation" => "NOT_EQUALS", "value" => "Y"]];
        }
        $data = $this->connectionService->find($this->dataSourceName, $params, false);
        $rows = array();

        $total = $this->count($params, false);
        $limit = array_key_exists('limit', $params) ? intval($params['limit']) : 99999;
        $page = array_key_exists('page', $params) ? intval($params['page']) : 1;

        foreach ($data as $row) {
            $row["usernameSsl"] = isset($row["usernameSsl"]) ? OpensslCrypt::decrypt($row["usernameSsl"]) : null;
            $row["dbUrlSsl"] = isset($row["dbUrlSsl"]) ? OpensslCrypt::decrypt($row["dbUrlSsl"]) : null;
            $row["username"] = $row["usernameSsl"];
            $row["dbUrl"] = $row["dbUrlSsl"] ;
            $row["passwordSsl"] = isset($row["passwordSsl"]) ? $row["passwordSsl"] : null ;
            $row["password"] = isset($row["password"]) ? $row["password"] : $row["passwordSsl"];
            $row["oracleWalletUse"] = isset($row["oracleWalletUse"]) ? $row["oracleWalletUse"] : false;

            $rows[] = $row;
        }
        $this->sendResponse(["data" => array(
	        "data" => $rows,
	        "WALLET_NRORG" => [Utilities::getOrganizationIdForMongo()],
	        "pagination" => ["total" => $total, "page" => $page, "limit" => $limit]
        )], 200);
	}
	
	public function count($params, $decodeBase64 = true) {
	    return $this->mongoManager->countNext($this->dataSourceName, $params, $decodeBase64);
	}

    public function findNextField(DTO\Request $request, DTO\Response $response)
    {
        $rows = array();
        $dataset = current($this->findData($request, $response)->getDataSets());
        foreach ($dataset->getRows() as $row) {
            $row["usernameSsl"] = isset($row["usernameSsl"]) ? OpensslCrypt::decrypt($row["usernameSsl"]) : null;
            $row["dbUrlSsl"] = isset($row["dbUrlSsl"]) ? OpensslCrypt::decrypt($row["dbUrlSsl"]) : null;
            $row["username"] = $row["usernameSsl"];
            $row["dbUrl"] = $row["dbUrlSsl"] ;
            $row["passwordSsl"] = isset($row["passwordSsl"]) ? $row["passwordSsl"] : null ;
            $row["password"] = isset($row["password"]) ? $row["password"] : $row["passwordSsl"];
            $row["oracleWalletUse"] = isset($row["oracleWalletUse"]) ? $row["oracleWalletUse"] : false;

            $rows[] = $row;
        }
        $this->sendResponse(Utilities::parseDataSourceZeedhiNext($rows), 200);
    }


    public function checkCreateNewConnectionNext(DTO\Request $request, DTO\Response $response) {
        //  parent::find($request, $response);
        $filter = Utilities::parseParamNextToZmartFilterCriteria($this->dataSourceName, $request->getParameters());
        $dataSet = $this->mongoManager->findBy($filter);
        $response->addDataSet($dataSet);
        $connections = current($response->getDataSets())->getRows();
        $rows=[];
        if(empty($connections)){
            $rows["hasConnection"] = false;
            $isOrgSaas = $this->connectionService->checkIfSaas();
            $rows["idOrgSaas"] = $isOrgSaas;
			if($isOrgSaas){
				$this->connectionService->createDefaultConnection();
			}
		}else{
			$rows["hasConnection"] = true;
		}
		$this->sendResponse(["data" => $rows], 200);
	}
	public function testConnection(DTO\Request $request, DTO\Response $response) {
		$row = $request->getRow();

		try{
			$testResponse = $this->tryConnection($row, true);
			$response->addDataSet(new DataSet('testResponse', $testResponse));
		} catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
	}

	public function testAllConnections(DTO\Request $request, DTO\Response $response) {
		parent::find($request, $response);

		$connections = current($response->getDataSets())->getRows();
		$testConnections = array();

		foreach ($connections as $connection) {
			$connectionName = $connection['name'];
			$testResponse = null;
			$hasError = false;

			try {
				$testResponse = $this->tryConnection($connection);

				if($testResponse['hasError']) {
					$hasError = true;
				}
			} catch (\Exception $e) {
				$hasError = true;
				$testResponse = $e->getMessage();
	        }

			$testConnection = array(
				"connection" => $connectionName,
                "testResult" => isset($testResponse['response'])?$testResponse['response']:$testResponse,
				"hasError"	 => $hasError
			);

			array_push($testConnections, $testConnection);
		}

		$response->setDataSets(array(new DataSet('testConnections', $testConnections)));
	}

	protected function tryConnection($row, $needEncrypt = null) {
		$clientVersion = null;
        if(isset($row["oracleWalletUse"]) && $row["driverClass"] == "oracle.jdbc.driver.OracleDriver" && $row["oracleWalletUse"] == 'Y'){
            $validate = Utilities::validateWallet($row["dbUrl"]);
            if(!$validate){
                throw new \Exception('ERROR_WALLET_SECURITY',1);
            }
        }
		if(isset($row["passwordNeedEncrypt"]) && $row["passwordNeedEncrypt"]) {
			$row["passwordSsl"] = OpensslCrypt::encrypt($row["password"]);
		}

		if(empty($row["passwordSsl"])){
            $row["passwordSsl"] = $row["password"];
        }

		$connectionsParams = $this->getConnectionParams($row, $needEncrypt);
		$response = $this->connectionService->testConnection($connectionsParams);
		return $this->validateResponse($response);
	}

	protected function validateResponse($response) {
		$hasError = isset($response['hasError']) ? $response['hasError'] : null;
		$defaultErrorMsg = isset($response['error']) ? $response['error'] : "Connection not succeeded";
		$return = $defaultErrorMsg;

		if(isset($response['data']['response'])) {
			$response = $response['data']['response'];

			if($hasError) {
				$return = $this->getResponseErrorMessage($response, $defaultErrorMsg);
			} else {
				$response = get_object_vars(json_decode($response));
				$return = isset($response['product']) ? get_object_vars($response['product']) : null;
			}
		}

		return array(
			'response' => $return,
			'hasError' => $hasError
		);
	}

	protected function getResponseErrorMessage($response, $defaultErrorMsg) {
		if(isset($response['sqlException']['detailMessage'])) {
			return $response['sqlException']['detailMessage'];
		} else if(isset($response['cause']['detailMessage'])) {
			return $response['cause']['detailMessage'];
		} else if(isset($response['detailMessage'])) {
			return $response['detailMessage'];
		} else {
			return $defaultErrorMsg;
		}
	}

	protected function getConnectionParams($row, $needEncrypt) {
		$organizationID = Utilities::getOrganizationIdForMongo();
		$clientRoom = $this->connectionService->getClientRoom($organizationID, $row);

		$dbUrl = $this->getDbUrl($row);
        $usernameSsl =  isset($row["usernameSsl"]) ? $row["usernameSsl"] : null;
        $dbUrlSsl =  isset($row["dbUrlSsl"]) ? $row["dbUrlSsl"] : null;

        return array(
            "dbUrlSsl" => $needEncrypt ? OpensslCrypt::encrypt($dbUrl) : $dbUrlSsl,
            "usernameSsl" => $needEncrypt ? OpensslCrypt::encrypt($row["username"]) : $usernameSsl,
            "password" => isset($row["password"]) ? $row["password"] : null,
            "passwordSsl" => isset($row["passwordSsl"]) ? $row["passwordSsl"] : null,
			"room" => $clientRoom,
            "query" => $this->getQuery(isset($row["value"]) ? $row["value"] : $row["database"]),
            "dialect" => isset($row['dialect']) ? $row['dialect'] : null,
            "driverClass" => isset($row['driverClass']) ? $row['driverClass'] : null,
            "oracleWalletUse" => isset($row['oracleWalletUse']) ? $row['oracleWalletUse'] : 'N'
		);
	}

	public function getQuery($database){
		switch ($database) {
			case 'sqlserver':
			case 'mysql':
				return 'SELECT 1';
			case 'oracle':
			default:
				return 'SELECT 1 FROM DUAL';
		}
	}

	protected function getDbUrl($row){
        $dbUrl = '';
		if(isset($row["dbUrl"])){
			$dbUrl = $row["dbUrl"];
		} else if(isset($row['host']) && isset($row['host']) && isset($row['port']) && isset($row['dbname'])){
			$dbUrl = "jdbc:oracle:thin:@".$row['host'].":".$row['port']."/".$row['dbname'];
		}

		return $dbUrl;
	}

    public function getUserCurrentIsTeknisa(DTO\Request $request, DTO\Response $response) {
		try{
			$response->addDataSet(new DataSet('IS_TEKNISA', [Utilities::userCurrentIsTeknisa()]));
		} catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
	}
}
